#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <math.h>

struct position {
  double x;
  double y;
};

#define TEMP_FILE "gps2gis.tmp"
#define MAX_TRACK 1024
#define MAX_POINT  512

int delete_colon(char *, char *);
int read_waypoint(char *, struct position *, char *, char *, FILE *);
int write_waypoint(int id, char *, struct position, char *, char *, FILE *, FILE *);
     int read_track_point(struct position *, FILE *);

/* Datei Zeichen fuer Zeichen lesen, jedes ',' durch ' ' ersetzen
 * Variablen:
 * in: Pointer auf offene Dateien, original Datei (lesen) in
 *     gpsorig_zeiger, Zieldatei (schreiben) in gpsneu_zeiger 
 * out: - */

int delete_colon(char *dateiname_ausgabe, char *dateiname_eingabe)
{
  char c;

  FILE *gpsneu_zeiger;
  FILE *gpsorig_zeiger;
  
  /* Datei zeichenweise, d.h. 1:1 einlesen */
  gpsneu_zeiger = fopen(dateiname_ausgabe,"w");
  gpsorig_zeiger = fopen(dateiname_eingabe,"r");
  if (gpsorig_zeiger == NULL)
    {
      printf("file not found\n");
      exit();
    }

  while(fscanf(gpsorig_zeiger, "%c", &c) != EOF)
    {
      if (c == ',')
	fprintf(gpsneu_zeiger, " "); 
      else
	fprintf(gpsneu_zeiger, "%c", c);
    }

  fclose(gpsneu_zeiger);
  fclose(gpsorig_zeiger);
  return(0);
}

int read_waypoint(char *name, struct position *wp, 
		   char *str1, char *str2, FILE *fp)
{
  fscanf(fp, "%*s %s %lf %lf %s %s", 
	 name, &wp->y, &wp->x, str1, str2);
  return 0;
}
	
int write_waypoint(int id, char *name, struct position wp, 
		   char *str1, char *str2, 
		   FILE *fp1, FILE *fp2)
{
  /* Datei fuer Gen2Shp */
  fprintf(fp1, "%i, %20.14f, %20.14f\n", id, wp.x, wp.y);

  /* Datei fuer Referenzierung ID, Name et al. */
  fprintf(fp2, "%i, %s, %s, %s\n", id, name, str1, str2);
  return 0;
}

int read_track_point(struct position *pos, FILE *file_read)
{
  int i;

  fscanf(file_read, "%*s %lf %lf %*s %*s %i", &pos->y, &pos->x, &i);
  return i;
}


int write_line(struct position tracks[MAX_POINT], int length, FILE *fp)
{
  int j;
  
  for (j=0; j<length; j++)
    fprintf(fp, "%20.13f, %20.13f\n", tracks[j].x, tracks[j].y);
  fprintf(fp, "END\n");
  return 0;
}


double distance(struct position p1, struct position p2)
{
  return sqrt((p1.x-p2.x)*(p1.x-p2.x) + (p1.y-p2.y)*(p1.y-p2.y));
}

int track_is_polygon(struct position track[MAX_POINT], int nr_points, double delta)
{
  double d;

  d = distance(track[0], track[nr_points-1]);
  return ((d<delta) && (nr_points >= 3));
}

int main(int argc, char *argv[])
{
  int id_wp, track_nr, point_nr, i, nr_poly;
  
  double delta = 4.5e-5;

  char id_str[100], name[100], str1[100], str2[100];
  FILE *file_read, *file_write_w, *file_write_id, *fp;
  char wayp_fn[100] = "";
  char id_fn[100] = "";
  char track_fn[100] = "";
  char poly_fn[100] = "";

  struct position pos;
  struct position tracks[MAX_TRACK][MAX_POINT];
  int track_length[MAX_TRACK];


  printf("gps2gis (c) 2001 Programmierpraktikum IfG\n");
  if (argc < 3)
    {
      printf("gps2gis <gps file> <arcview-name> (<delta>)\n");
      printf(" the following files will be generated:\n");
      printf("  <arcview-name>.txt list of waypoint names and identifers\n");
      printf("  <arcview-name>.way Gen2Shp file for waypoint import\n");
      printf("  <arcview-name>.track Gen2Shp file for track-log import (lines)\n");
      printf("  <arcview-name>.poly Gen2Shp file for track-log import (polygons)\n");
      printf(" optional, if <delta> walue given, it overrides the default value\n");
      printf(" of GPS accuracy for closing polygons.\n");
      exit(0);
    }

  if (argc == 4)
    sscanf(argv[3], "%lf", &delta);

  delete_colon(TEMP_FILE, argv[1]);

  file_read = fopen(TEMP_FILE, "r");

  strcpy(id_fn, argv[2]); strcat(id_fn, ".txt");
  strcpy(wayp_fn, argv[2]); strcat(wayp_fn, ".way");
  strcpy(track_fn, argv[2]); strcat(track_fn, ".track");
  strcpy(poly_fn, argv[2]); strcat(poly_fn, ".poly");

  file_write_w = fopen(wayp_fn, "w");
  file_write_id = fopen(id_fn, "w");
  fprintf(file_write_id, "id, name, remark 1, remark 2\n");

  id_wp = 0;
  track_nr = 0;
  point_nr = 0;
  
  while (fscanf(file_read, "%s", id_str) != EOF)
    { 		
      if (id_str[0] == 'w')
	{	
	  read_waypoint(name, &pos, str1, str2, file_read);
	  write_waypoint(id_wp, name, pos, str1, str2, 
			 file_write_w, file_write_id);
	  id_wp++;
	}
      if (id_str[0] == 't')
	{	
	  if (read_track_point(&tracks[track_nr][point_nr++], file_read))
	    {
	      tracks[track_nr+1][0] = tracks[track_nr][--point_nr];
	      track_length[track_nr] = point_nr;
	      point_nr = 1;
	      track_nr++;
	    }
	}
    }

  fprintf(file_write_w, "END\n");
  fclose(file_write_w);
  fclose(file_read);
  remove(TEMP_FILE);

  fp = fopen(track_fn, "w");
  for (i=1; i<track_nr; i++)
    {
      fprintf(fp, "%d\n", i);
      write_line(tracks[i], track_length[i], fp);
    }
  fprintf(fp, "END\n");
  fclose(fp);

  fp = fopen(poly_fn, "w");
  fprintf(fp, "CLOSE\n");
  nr_poly = 0;
  for (i=1; i<track_nr; i++)
    if (track_is_polygon(tracks[i], track_length[i], delta))
      {
	fprintf(fp, "%d,", i);
	write_line(tracks[i], track_length[i], fp); 
	nr_poly++;
      }
  fprintf(fp, "END\n");
  fclose(fp);

  printf("%d waypoints in %s \n", id_wp, wayp_fn);
  printf("%d tracks in %s\n", track_nr, track_fn);
  printf("%d polygons in %s\n", nr_poly, poly_fn);
  return(0);
}	
